package pay.platform.mapper;

import com.mybatisflex.core.BaseMapper;
import com.mybatisflex.core.query.QueryWrapper;
import org.apache.ibatis.annotations.Mapper;
import org.apache.ibatis.annotations.Param;
import pay.platform.domain.SysRolePermission;
import pay.platform.domain.table.SysRolePermissionTableDef;

import java.util.List;

/**
 * 系统 - 角色权限 映射层。
 *
 * @author admin
 * @since 1.0
 */
@Mapper
public interface SysRolePermissionMapper extends BaseMapper<SysRolePermission> {

    /**
     * 根据权限ID获取所有的父级ID
     *
     * @param permissionIds .
     * @return .
     */
    default List<String> getParentIdByPermissionIds(@Param("permissionIds") List<String> permissionIds) {
        QueryWrapper query = new QueryWrapper()
                .select(SysRolePermissionTableDef.SYS_ROLE_PERMISSION.PERMISSION_ID)
                .from(SysRolePermissionTableDef.SYS_ROLE_PERMISSION)
                .where(SysRolePermissionTableDef.SYS_ROLE_PERMISSION.ID.in(permissionIds));
        return selectListByQueryAs(query, String.class).stream().distinct().toList();
    }

    /**
     * 根据角色ID查询权限ID,只查询没有下级的权限
     *
     * @param roleId .
     * @return .
     */
    default List<String> getPermissionIdByRoleId(@Param("roleId") String roleId) {
        QueryWrapper query = new QueryWrapper()
                .select(SysRolePermissionTableDef.SYS_ROLE_PERMISSION.PERMISSION_ID)
                .from(SysRolePermissionTableDef.SYS_ROLE_PERMISSION)
                .where(SysRolePermissionTableDef.SYS_ROLE_PERMISSION.ROLE_ID.eq(roleId));
//                .eq(new QueryWrapper().select("count(id)")
//                        .from(SysPermissionTableDef.SYS_PERMISSION).where(SysPermissionTableDef.SYS_PERMISSION.PARENT_ID.eq(SysRolePermissionTableDef.SYS_ROLE_PERMISSION.PERMISSION_ID)).toSQL(), 0);
        return selectListByQueryAs(query, String.class);
    }
}
