package pay.platform.api.system.controller;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.StrUtil;
import cn.monitor4all.logRecord.annotation.OperationLog;
import cn.monitor4all.logRecord.context.LogRecordContext;
import com.alibaba.fastjson2.JSONObject;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.Parameter;
import io.swagger.v3.oas.annotations.Parameters;
import io.swagger.v3.oas.annotations.tags.Tag;
import jakarta.servlet.http.HttpServletRequest;
import org.springdoc.core.annotations.ParameterObject;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;
import pay.platform.api.system.model.query.PayAgentQuery;
import pay.platform.api.system.model.vo.PayAgentVO;
import pay.platform.api.system.model.vo.RestPasswdVO;
import pay.platform.api.system.servcie.PayAgentService;
import pay.platform.core.common.Result;
import pay.platform.core.common.domain.BasePage;
import pay.platform.domain.PayAgent;

import java.io.Serializable;
import java.util.List;

/**
 * 代理 控制层。
 *
 * @author mybatis-flex-helper automatic generation
 * @since 1.0
 */
@RestController
@RequestMapping("/sys/pay/agent")
@Tag(name = "代理控制层")
public class PayAgentController {

    @Autowired
    private PayAgentService payAgentService;

    /**
     * 添加 代理
     *
     * @param PayAgent 代理
     * @return {@code true} 添加成功，{@code false} 添加失败
     */
    @PostMapping("/save")
    @Operation(summary = "添加代理")
    @OperationLog(bizType = "'添加代理'", bizId = "#PayChannel.userName", msg = "'用户' + #userName + '添加了通道 #info")
    public Result<String> save(@RequestBody PayAgent PayAgent) {
        LogRecordContext.putVariable("info", JSONObject.toJSONString(PayAgent));

        return payAgentService.saveAgent(PayAgent);
    }


    /**
     * 根据主键删除代理
     *
     * @param id 主键
     * @return {@code true} 删除成功，{@code false} 删除失败
     */
    @DeleteMapping("/remove/{id}")
    @Operation(summary = "根据主键删除代理")
    @Parameters(value = {
            @Parameter(name = "id", description = "id", required = true)
    })
    @OperationLog(bizType = "'删除代理'", bizId = "#id", msg = "'原值:' + #orgInfo")
    public Result<Boolean> remove(@PathVariable Serializable id) {
        LogRecordContext.putVariable("orgInfo", JSONObject.toJSONString(payAgentService.getById(id)));

        return Result.OK(payAgentService.removeById(id));
    }


    /**
     * 根据主键更新代理
     *
     * @param PayAgent 代理
     * @return {@code true} 更新成功，{@code false} 更新失败
     */
    @PutMapping("/update")
    @Operation(summary = "根据主键更新代理")
    @OperationLog(bizType = "'更新代理'", bizId = "#PayChannel.name", msg = "'原值:' + #orgInfo + '修改后的值:' +#updateInfo")

    public Result<Boolean> update(@RequestBody PayAgent PayAgent) {
        LogRecordContext.putVariable("orgInfo", JSONObject.toJSONString(payAgentService.getById(PayAgent.getId())));
        LogRecordContext.putVariable("updateInfo", JSONObject.toJSONString(PayAgent));

        return Result.OK(payAgentService.updateInfo(PayAgent));
    }


    /**
     * 查询所有代理
     *
     * @return 所有数据
     */
    @GetMapping("/list")
    @Operation(summary = "查询所有代理")
    public Result<List<PayAgentVO>> list() {
        var list = payAgentService.list();
        List<PayAgentVO> vos = BeanUtil.copyToList(list, PayAgentVO.class);

        return Result.OK(vos);
    }


    /**
     * 根据代理主键获取详细信息。
     *
     * @param id PayAgent主键
     * @return 代理详情
     */
    @GetMapping("/getInfo/{id}")
    @Operation(summary = "根据代理主键获取详细信息")
    @Parameters(value = {
            @Parameter(name = "id", description = "id", required = true)
    })
    public Result<PayAgent> getInfo(@PathVariable Serializable id) {
        return Result.OK(payAgentService.getById(id));
    }


    /**
     * 分页查询代理
     *
     * @param query 分页对象
     * @return 分页对象
     */
    @GetMapping("/page")
    @Operation(summary = "分页查询代理")
    public Result<BasePage<PayAgentVO>> page(@ParameterObject PayAgentQuery query) {
        return Result.OK(payAgentService.queryPage(query));
    }

    @PutMapping("/reset/passwd/{username}")
    @Operation(summary = "重置密码")
    public Result<String> resetPasswd(HttpServletRequest request, @PathVariable(value = "username") String username,
                                      @Validated @RequestBody RestPasswdVO vo) {
        if (StrUtil.isBlank(vo.getNewPassword())) {
            return Result.NG("新密码不能为空");
        }
        if (!vo.getNewPassword().equals(vo.getConfirmPassword())) {
            return Result.NG("两次密码不一致");
        }
        return payAgentService.resetPassword(username, vo.getConfirmPassword());
    }
}