package pay.platform.api.system.servcie;


import cn.hutool.core.util.StrUtil;
import com.mybatisflex.core.paginate.Page;
import com.mybatisflex.core.query.QueryWrapper;
import pay.platform.api.system.model.query.RoleQuery;
import pay.platform.core.common.Result;
import pay.platform.core.common.domain.BasePage;
import pay.platform.core.common.domain.BaseService;
import pay.platform.domain.SysRole;
import pay.platform.domain.table.SysRoleTableDef;
import pay.platform.mapper.SysRoleMapper;
import jakarta.annotation.Nonnull;
import jakarta.annotation.Nullable;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;

/**
 * 系统 - 角色信息 服务层实现。
 *
 * @author ADMIN
 * @since 1.0
 */
@Service
@Slf4j
@RequiredArgsConstructor
public class SysRoleService extends BaseService<SysRoleMapper, SysRole> {

    /**
     * 分页查询
     *
     * @param query .
     * @return .
     */
    public BasePage<SysRole> queryPage(RoleQuery query) {
        QueryWrapper queryWrapper = buildQuery(query);
        Page<SysRole> page = new Page<>(query.getCurrent(), query.getSize());
        page = page(page, queryWrapper);
        return new BasePage<>(page.getPageNumber(), page.getPageSize(), page.getTotalRow(), page.getRecords());
    }

    /**
     * 列表查询
     *
     * @param query .
     * @return .
     */
    public List<SysRole> queryList(RoleQuery query) {
        QueryWrapper queryWrapper = buildQuery(query);
        return list(queryWrapper);
    }

    /**
     * 检查code是否重复
     *
     * @param code .
     * @param id   需要排查的ID，可以为空
     * @return .
     */
    public boolean checkCode(@Nonnull String code, @Nullable String id) {
        QueryWrapper queryWrapper = QueryWrapper.create().from(SysRoleTableDef.SYS_ROLE).eq(SysRole::getRoleCode, code).ne(SysRole::getId, id, StrUtil.isNotBlank(id));
        return count(queryWrapper) > 0;
    }

    /**
     * 保存
     *
     * @param sysRole .
     * @return .
     */
    @Transactional(rollbackFor = Exception.class)
    public Result<SysRole> saveRole(SysRole sysRole) {
        if (StrUtil.isBlank(sysRole.getRoleName()) || StrUtil.isBlank(sysRole.getRoleCode())) {
            return Result.NG("参数错误,请检查参数");
        }
        boolean checkCode = checkCode(sysRole.getRoleCode(), null);
        if (checkCode) {
            return Result.NG("角色编码: " + sysRole.getRoleCode() + "重复");
        }
        save(sysRole);
        return Result.OK(sysRole);
    }

    /**
     * 更新角色
     *
     * @param sysRole .
     * @return .
     */
    public Result<SysRole> updateRole(SysRole sysRole) {
        if (StrUtil.isBlank(sysRole.getRoleName()) || StrUtil.isBlank(sysRole.getRoleCode())) {
            return Result.NG("参数错误,请检查参数");
        }
        boolean checkCode = checkCode(sysRole.getRoleCode(), sysRole.getId());
        if (checkCode) {
            return Result.NG("角色编码: " + sysRole.getRoleCode() + "重复");
        }
        updateById(sysRole);
        return Result.OK(sysRole);
    }

    private QueryWrapper buildQuery(RoleQuery query) {
        QueryWrapper queryWrapper = QueryWrapper.create().from(SysRoleTableDef.SYS_ROLE);
        if (null == query) {
            return queryWrapper;
        }
        queryWrapper.eq(SysRole::getRoleCode, query.getRoleCode(), StrUtil.isNotBlank(query.getRoleCode()));
        queryWrapper.like(SysRole::getRoleName, query.getRoleName(), StrUtil.isNotBlank(query.getRoleName()));
        queryWrapper.eq(SysRole::getEnable, query.getEnable(), query.getEnable() != null);
        return queryWrapper;
    }
}