package pay.platform.api.system.servcie.impl;


import cn.hutool.core.bean.BeanUtil;
import com.mybatisflex.core.paginate.Page;
import com.mybatisflex.core.query.QueryWrapper;
import com.mybatisflex.core.util.StringUtil;
import jakarta.annotation.Nonnull;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;
import pay.platform.api.system.model.query.PayAgentQuery;
import pay.platform.api.system.model.vo.PayAgentVO;
import pay.platform.api.system.servcie.PayAgentService;
import pay.platform.core.common.Result;
import pay.platform.core.common.domain.BasePage;
import pay.platform.core.common.domain.BaseService;
import pay.platform.core.util.PasswordUtil;
import pay.platform.domain.PayAgent;
import pay.platform.domain.table.PayAgentTableDef;
import pay.platform.mapper.PayAgentMapper;

import java.util.List;

import static pay.platform.domain.table.PayAgentTableDef.PAY_AGENT;

/**
 * 代理系统 服务层实现。
 *
 * @author mybatis-flex-helper automatic generation
 * @since 1.0
 */
@Service
public class PayAgentServiceImpl extends BaseService<PayAgentMapper, PayAgent> implements PayAgentService {

    @Override
    public Result<String> saveAgent(PayAgent payAgent) {
        if (StringUtils.isBlank(payAgent.getPassword())) {
            return Result.NG("密码不能为空");
        }
        QueryWrapper queryWrapper = QueryWrapper.create().from(PAY_AGENT).where(PAY_AGENT.USER_NAME.eq(payAgent.getUserName()));
        if (count(queryWrapper) > 0) {
            return Result.NG("代理账号已存在");
        }
        payAgent.setPassword(PasswordUtil.encoder(payAgent.getPassword()));
        save(payAgent);
        return Result.OK();
    }

    @Override
    public BasePage<PayAgentVO> queryPage(PayAgentQuery query) {
        QueryWrapper queryWrapper = QueryWrapper.create().from(PayAgentTableDef.PAY_AGENT).where("1 = 1")
                .and(PayAgentTableDef.PAY_AGENT.USER_NAME.like(query.getUserName(), StringUtil.isNotBlank(query.getUserName())))
                .and(PAY_AGENT.STATUS.eq(query.getStatus(), query.getStatus() != null));
        Page<PayAgent> page = mapper.paginate(new Page<>(query.getCurrent(), query.getSize()), queryWrapper);
        List<PayAgentVO> vos = BeanUtil.copyToList(page.getRecords(), PayAgentVO.class);
        return new BasePage<>(page.getPageNumber(), page.getPageSize(), page.getTotalRow(), vos);
    }

    /**
     * 重置密码
     *
     * @param username .
     * @param passwd   .
     * @return .
     */

    public Result<String> resetPassword(@Nonnull String username, @Nonnull String passwd) {
        String newPassword = PasswordUtil.encoder(passwd);
        mapper.resetPassword(username, newPassword);
        return Result.OK();
    }

    @Override
    public Boolean updateInfo(PayAgent payAgent) {
        if (StringUtils.isNotBlank(payAgent.getPassword())) {
            payAgent.setPassword(PasswordUtil.encoder(payAgent.getPassword()));
        }
        return this.updateById(payAgent);
    }
}