package pay.platform.core.springdoc;

import pay.platform.core.common.PlatformConst;
import io.swagger.v3.oas.models.Components;
import io.swagger.v3.oas.models.OpenAPI;
import io.swagger.v3.oas.models.info.Contact;
import io.swagger.v3.oas.models.info.Info;
import io.swagger.v3.oas.models.security.SecurityRequirement;
import io.swagger.v3.oas.models.security.SecurityScheme;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;

/**
 * Springdoc配置
 */
@Configuration
public class SpringdocConfiguration {
    @Bean
    public OpenAPI openAPI() {
        return new OpenAPI()
                .info(getInfo())
                .addSecurityItem(new SecurityRequirement().addList(PlatformConst.X_ACCESS_TOKEN))
                .schemaRequirement(PlatformConst.X_ACCESS_TOKEN, securityScheme())
                .components(new Components().addSecuritySchemes(PlatformConst.X_ACCESS_TOKEN, securityScheme()));
    }

    private Info getInfo() {
        return new Info()
                .title("API")
                .contact(new Contact().name("ADMIN").url("http://localhost:8088"))
                .version("v1.0.0");
    }

    private SecurityScheme securityScheme() {
        SecurityScheme securityScheme = new SecurityScheme();
        securityScheme.setType(SecurityScheme.Type.APIKEY);
        securityScheme.setName(PlatformConst.X_ACCESS_TOKEN);
        securityScheme.setIn(SecurityScheme.In.HEADER);
        return securityScheme;
    }
}
