package pay.platform.api.system.controller;

import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.springdoc.core.annotations.ParameterObject;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import pay.platform.api.system.model.query.PayOrderQuery;
import pay.platform.api.system.model.vo.HomeVO;
import pay.platform.api.system.model.vo.PayOrderVO;
import pay.platform.api.system.servcie.PayOrderService;
import pay.platform.core.common.Result;
import pay.platform.core.common.domain.BasePage;

import java.util.List;

/**
 * 订单 控制层。
 *
 * @author mybatis-flex-helper automatic generation
 * @since 1.0
 */
@RestController
@RequestMapping("/agent/pay/order")
@Tag(name = "订单控制层")
public class PayOrderController {

    @Autowired
    private PayOrderService PayOrderService;


    /**
     * 订单统计
     * @param query
     * @return
     */
    @GetMapping("/orderTotal")
    @Operation(summary = "订单统计")
    public Result<List<HomeVO>> orderTotal(@ParameterObject PayOrderQuery query) {
        var payOrder = PayOrderService.orderTotal(query);
        return Result.OK(payOrder);
    }

    /**
     * 订单分组统计
     *
     * @return Result.OK(所有数据
     */
    @GetMapping("/group/total")
    @Operation(summary = "订单分组统计")
    public Result<List<PayOrderVO>> orderTotalGroup(PayOrderQuery payOrderQuery) {
        return Result.OK(PayOrderService.orderTotalGroup(payOrderQuery));
    }


    /**
     * 分页查询订单
     *
     * @param query 分页对象
     * @return Result<分页对象
     */
    @GetMapping("/page")
    @Operation(summary = "分页查询订单")
    public Result<BasePage<PayOrderVO>> page(@ParameterObject PayOrderQuery query) {
        return Result.OK(PayOrderService.queryPage(query));
    }
}