package pay.platform.api.system.servcie.impl;


import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.RandomUtil;
import com.mybatisflex.core.paginate.Page;
import com.mybatisflex.core.query.QueryWrapper;
import com.mybatisflex.core.util.StringUtil;
import jakarta.annotation.Nonnull;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import pay.platform.api.system.model.query.PayMerchantQuery;
import pay.platform.api.system.model.vo.PayMerchantVO;
import pay.platform.api.system.servcie.PayMerchantAppKeyService;
import pay.platform.api.system.servcie.PayMerchantService;
import pay.platform.core.common.Result;
import pay.platform.core.common.domain.BasePage;
import pay.platform.core.common.domain.BaseService;
import pay.platform.core.security.util.SecurityUtil;
import pay.platform.core.util.PasswordUtil;
import pay.platform.domain.PayMerchant;
import pay.platform.domain.PayMerchantAppKey;
import pay.platform.mapper.PayMerchantMapper;

import java.util.List;

import static pay.platform.domain.table.PayMerchantTableDef.PAY_MERCHANT;

/**
 * 商户管理 服务层实现。
 *
 * @author mybatis-flex-helper automatic generation
 * @since 1.0
 */
@Service
public class PayMerchantServiceImpl extends BaseService<PayMerchantMapper, PayMerchant> implements PayMerchantService {

    @Autowired
    private PayMerchantAppKeyService payMerchantAppKeyService;

    @Override
    @Transactional
    public Result<String> saveMerchant(PayMerchant payMerchant) {
        if (StringUtils.isBlank(payMerchant.getPassword())) {
            return Result.NG("密码不能为空");
        }
        payMerchant.setPassword(PasswordUtil.encoder(payMerchant.getPassword()));
        payMerchant.setAgentId(SecurityUtil.getCurrentUser().getUserid());
        QueryWrapper queryWrapper = QueryWrapper.create().from(PAY_MERCHANT).where(PAY_MERCHANT.USER_NAME.eq(payMerchant.getUserName()));
        if (count(queryWrapper) > 0) {
            return Result.NG("商户账号已存在");
        }
        save(payMerchant);
        PayMerchantAppKey payMerchantAppKey = PayMerchantAppKey.builder().appId(RandomUtil.randomString(30)).appSecret(RandomUtil.randomString(50))
                .appKey(RandomUtil.randomString(40)).agentId(SecurityUtil.getCurrentUser().getUserid())
                .merchantId(payMerchant.getId()).build();
        payMerchantAppKeyService.save(payMerchantAppKey);
        return Result.OK();
    }


    @Override
    public BasePage<PayMerchantVO> queryPage(PayMerchantQuery query) {
        QueryWrapper queryWrapper = QueryWrapper.create().from(PAY_MERCHANT).where("1 = 1")
                .and(PAY_MERCHANT.USER_NAME.like(query.getUserName(), StringUtil.isNotBlank(query.getUserName())))
                .and(PAY_MERCHANT.AGENT_ID.eq(SecurityUtil.getCurrentUser().getUserid(), StringUtil.isNotBlank(SecurityUtil.getCurrentUser().getUserid()))

                        .and(PAY_MERCHANT.STATUS.eq(query.getStatus(), query.getStatus() != null)));
        Page<PayMerchant> page = mapper.paginate(new Page<>(query.getCurrent(), query.getSize()), queryWrapper);
        List<PayMerchantVO> vos = BeanUtil.copyToList(page.getRecords(), PayMerchantVO.class);
        return new BasePage<>(page.getPageNumber(), page.getPageSize(), page.getTotalRow(), vos);
    }


    /**
     * 重置密码
     *
     * @param username .
     * @param passwd   .
     * @return .
     */

    public Result<String> resetPassword(@Nonnull String username, @Nonnull String passwd) {
        String newPassword = PasswordUtil.encoder(passwd);
        mapper.resetPassword(username, newPassword);
        return Result.OK();
    }

    @Override
    public Boolean updateInfo(PayMerchant payMerchant) {
        if (StringUtils.isNotBlank(payMerchant.getPassword())) {
            payMerchant.setPassword(PasswordUtil.encoder(payMerchant.getPassword()));
        }
        return this.updateById(payMerchant);
    }
}