package pay.platform.api.system.event;

import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.context.ApplicationListener;
import org.springframework.stereotype.Component;
import pay.platform.api.system.cache.RouteCache;
import pay.platform.core.security.token.RedisCacheUserProvider;

/**
 * 登出事件
 */
@Component
@RequiredArgsConstructor
@Slf4j
public class LogoutListener implements ApplicationListener<LogoutEvent> {

    private final RedisCacheUserProvider userCacheProvider;

    private final RouteCache routeCache;

    @Override
    public void onApplicationEvent(LogoutEvent event) {
        log.info("【用户注销事件】用户注销清理缓存信息>>>>>>>>>>>开始");
        String username = event.getUsername();
        userCacheProvider.clearUser(username);
        log.info("【用户注销事件】成功清理缓存登录用户信息");
        routeCache.removeCache(event.getUserid());
        log.info("【用户注销事件】成功清理缓存用户路由信息");
    }
}
