package pay.platform.api.controller;

import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import pay.platform.api.model.request.OrderRequest;
import pay.platform.api.model.request.PayRequest;
import pay.platform.api.model.request.PayTestRequest;
import pay.platform.api.model.response.OrderResponse;
import pay.platform.api.model.response.PayResponse;
import pay.platform.api.servcie.PayOrderService;
import pay.platform.api.servcie.PayService;
import pay.platform.core.common.Result;
import pay.platform.core.utils.SignUtil;

import java.util.TreeMap;

/**
 * 接口 - 支付
 */
@Slf4j
@RestController
@RequestMapping("/api")
@RequiredArgsConstructor
public class PayController {

    @Autowired
    private PayService payService;
    @Autowired
    private PayOrderService payOrderService;
    @PostMapping("/pay/alipay")
    public Result<PayResponse> pay(@RequestBody PayRequest payRequest) {
        log.info("payRequest:{}", payRequest);
        return Result.OK(payService.pay(payRequest));

    }

    @PostMapping("/pay/alipay/test")
    public Result<PayResponse> payTest(@RequestBody PayTestRequest payRequest) {
        log.info("payRequest:{}", payRequest);
        return Result.OK(payService.payTest(payRequest));

    }


    @PostMapping("/query/order")
    public Result<OrderResponse> orderQuery(@RequestBody OrderRequest orderRequest) {
        log.info("orderRequest:{}", orderRequest);
        return Result.OK(payOrderService.queryOrderByPlatformOrder(orderRequest));
    }

    public static void main(String[] args) {
//        TreeMap<String, String> treeMap = new TreeMap<>();
//        treeMap.put("orderNo", "2");
//        treeMap.put("amount", "0.01");
//        treeMap.put("subject", "购买商品");
//        treeMap.put("body", "测试下单");
//        treeMap.put("currentTime", "12345689");
//        treeMap.put("notifyUrl", "http://localhost:8091/api/return");
//        treeMap.put("appId", "1HHU21Or5S36ggww3DGyrjXgjHxxkC");
//        System.out.println(SignUtil.getSign("7nZZXJx1xqFv33OP88pmFVIAYsowgGk0QNbaP5al", treeMap));

        TreeMap<String, String> treeMap = new TreeMap<>();
        treeMap.put("orderNo", "1716881332886");
        treeMap.put("currentTime", "1716881332886");
        treeMap.put("appId", "1HHU21Or5S36ggww3DGyrjXgjHxxkC");
        System.out.println(SignUtil.getSign("7nZZXJx1xqFv33OP88pmFVIAYsowgGk0QNbaP5al", treeMap));
    }
}
