package pay.platform.api.servcie.impl;


import cn.hutool.http.HttpUtil;
import com.alibaba.fastjson2.JSONObject;
import com.mybatisflex.core.query.QueryWrapper;
import org.jetbrains.annotations.NotNull;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Service;
import pay.platform.api.model.request.OrderRequest;
import pay.platform.api.model.response.OrderResponse;
import pay.platform.api.servcie.PayMerchantAppKeyService;
import pay.platform.api.servcie.PayOrderService;
import pay.platform.core.common.domain.BaseService;
import pay.platform.core.exception.BusinessException;
import pay.platform.core.utils.SignUtil;
import pay.platform.domain.PayMerchantAppKey;
import pay.platform.domain.PayOrder;
import pay.platform.domain.table.PayOrderTableDef;
import pay.platform.mapper.PayOrderMapper;

import java.util.TreeMap;

/**
 * 通道管理 服务层实现。
 *
 * @author mybatis-flex-helper automatic generation
 * @since 1.0
 */
@Service
public class PayOrderServiceImpl extends BaseService<PayOrderMapper, PayOrder> implements PayOrderService {

    @Autowired
    private PayMerchantAppKeyService payMerchantAppKeyService;

    @Override
    public OrderResponse queryOrderByPlatformOrder(OrderRequest orderRequest) {
        PayMerchantAppKey payMerchantAppKey = payMerchantAppKeyService.getByAppId(orderRequest.getAppId());
        if (payMerchantAppKey == null) {
            throw new BusinessException("appId不存在");
        }
        //验证签名
        validSign(orderRequest, payMerchantAppKey);
        QueryWrapper queryWrapper = QueryWrapper.create().from(PayOrderTableDef.PAY_ORDER).where(PayOrderTableDef.PAY_ORDER.ORDER_NO.eq(orderRequest.getOrderNo()))
                .and(PayOrderTableDef.PAY_ORDER.MERCHANT_ID.eq(payMerchantAppKey.getMerchantId()));
        PayOrder payOrder = mapper.selectOneByQuery(queryWrapper);
        if (payOrder == null) {
            throw new BusinessException("订单不存在");
        }
        var orderResponse = getOrderResponse(payOrder, payMerchantAppKey);
        return orderResponse;

    }

    private @NotNull OrderResponse getOrderResponse(PayOrder payOrder, PayMerchantAppKey payMerchantAppKey) {
        OrderResponse orderResponse = OrderResponse.builder().orderNo(payOrder.getOrderNo()).amount(payOrder.getAmount()).status(payOrder.getStatus()).currentTime(System.currentTimeMillis()).build();
        orderResponse.setSign(orderResponseSign(orderResponse, payMerchantAppKey));
        return orderResponse;
    }

    @Override
    public PayOrder queryOrderByPlatformOrder(String merchantId, String orderNo) {
        QueryWrapper queryWrapper = QueryWrapper.create().from(PayOrderTableDef.PAY_ORDER).where(PayOrderTableDef.PAY_ORDER.PLATFORM_ORDER_NO.eq(orderNo))
                .and(PayOrderTableDef.PAY_ORDER.MERCHANT_ID.eq(merchantId));
        return mapper.selectOneByQuery(queryWrapper);
    }

    @Override
    public PayOrder queryByOrderNo(String merchantId, String orderNo) {
        QueryWrapper queryWrapper = QueryWrapper.create().from(PayOrderTableDef.PAY_ORDER).where(PayOrderTableDef.PAY_ORDER.ORDER_NO.eq(orderNo))
                .and(PayOrderTableDef.PAY_ORDER.MERCHANT_ID.eq(merchantId));
        return mapper.selectOneByQuery(queryWrapper);
    }


    /**
     * 回调签名
     *
     * @param payOrder
     * @param payMerchantAppKey
     */
    private String orderResponseSign(OrderResponse payOrder, PayMerchantAppKey payMerchantAppKey) {
        TreeMap<String, String> treeMap = new TreeMap<>();
        treeMap.put("orderNo", payOrder.getOrderNo());
        treeMap.put("amount", payOrder.getAmount().toString());
        treeMap.put("status", payOrder.getStatus().toString());
        treeMap.put("currentTime", String.valueOf(payOrder.getCurrentTime()));
        return SignUtil.getSign(payMerchantAppKey.getAppKey(), treeMap);
    }

    /**
     * 回调商户
     *
     * @param payOrder
     * @param payMerchantAppKey
     */
    @Async
    public void notifyMerchant(PayOrder payOrder, PayMerchantAppKey payMerchantAppKey) {
        var orderResponse = getOrderResponse(payOrder, payMerchantAppKey);
        var post = HttpUtil.post(payOrder.getNotifyUrl(), JSONObject.toJSONString(orderResponse));
        if (post.contains("success")) {
            payOrder.setNotifyCount(1);
            payOrder.setNotifyStatus(1);
            updateById(payOrder);
        } else {
            payOrder.setNotifyCount(1);
            updateById(payOrder);
        }
    }

    private void validSign(OrderRequest orderRequest, PayMerchantAppKey payMerchantAppKey) {
        TreeMap<String, String> treeMap = new TreeMap<>();
        treeMap.put("orderNo", orderRequest.getOrderNo());
        treeMap.put("appId", orderRequest.getAppId());
        treeMap.put("currentTime", orderRequest.getCurrentTime());
        if (!orderRequest.getSign().equals(SignUtil.getSign(payMerchantAppKey.getAppKey(), treeMap))) {
            throw new BusinessException("签名错误");
        }
    }
}